import typing
from abc import ABCMeta, abstractmethod, abstractproperty


class ConfigProvider(metaclass=ABCMeta):
    @abstractproperty
    def provider_code(self) -> str:
        pass

    @abstractmethod
    def get_variable(self, name: str) -> typing.Union[typing.Any, None]:
        pass

    @abstractmethod
    def get_variables_list(self) -> typing.List[typing.Any]:
        pass


class ConfigProviderManager:
    @staticmethod
    def get_by_code(code: str) -> typing.Union[ConfigProvider, None]:
        for provider in ConfigProvider.__subclasses__():
            if provider.provider_code == code:
                return provider



import typing

from .config.providers import ConfigProvider, ConfigProviderManager
from .credentials.providers import CredentialProvider, CredentialProviderManager


class Sitri:
    def __init__(self, credential_provider: CredentialProvider = None, config_provider: ConfigProvider = None):

        if not credential_provider or not config_provider:
            raise RuntimeError("Provider not found!")

        self.credential_provider = credential_provider
        self.config_provider = config_provider

    def get_credential(self, name: str, default: typing.Any = None) -> typing.Union[typing.Any, None]:
        variable = self.credential_provider.get_credential(name)

        return variable if variable else default

    def get_config(self, name: str, default: typing.Any = None) -> typing.Union[typing.Any, None]:
        variable = self.config_provider.get_variable(name)

        return variable if variable else default



import os
import typing

from .config.providers import ConfigProvider
from .credentials.providers import CredentialProvider


class SystemCredentialProvider(CredentialProvider):
    _project_prefix = "system"
    provider_code = "system"

    def __init__(self, project_prefix: str):
        self._project_prefix = project_prefix.upper()

    def prefixize(self, varname: str) -> str:
        return f"{self._project_prefix}_{varname.upper()}"

    def get_credential(self, identifier: str) -> typing.Union[str, None]:
        if identifier in os.environ:
            return os.getenv(self.prefixize(identifier))


class SystemConfigProvider(ConfigProvider):
    _project_prefix = "system"
    provider_code = "system"

    def __init__(self, project_prefix: str):
        self._project_prefix = project_prefix.upper()

    def prefixize(self, varname: str) -> str:
        return f"{self._project_prefix}_{varname.upper()}"

    def get_variable(self, name: str) -> typing.Union[str, None]:
        return os.getenv(self.prefixize(name), None)

    def get_variables_list(self) -> typing.List[str]:
        var_list = []

        for var in os.environ:
            if self._project_prefix in var:
                var_list.append(var)

        return var_list



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['sitri', 'sitri.config', 'sitri.credentials']

package_data = \
{'': ['*']}

install_requires = \
['pytest>=5.1,<6.0']

setup_kwargs = {
    'name': 'sitri',
    'version': '0.1.0',
    'description': 'Library for one endpoint config and credentials managment',
    'long_description': None,
    'author': 'Alexander Lavrov',
    'author_email': 'egnod@ya.ru',
    'url': 'https://github.com/Egnod/sitri',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import typing
from abc import ABCMeta, abstractmethod, abstractproperty


class CredentialProvider(metaclass=ABCMeta):
    @abstractproperty
    def provider_code(self) -> str:
        pass

    @abstractmethod
    def get_credential(self, identifier: str) -> typing.Any:
        pass


class CredentialProviderManager:
    @staticmethod
    def get_by_code(code: str) -> typing.Union[CredentialProvider, None]:
        for provider in CredentialProvider.__subclasses__():
            if provider.provider_code == code:
                return provider


# Simple example with Vault by HashiCorp
"""
from ..config import get_config
import hvac

class VaultProvider(CredentialProvider):
    provider_code = "vault"

    def __init__(self, token: str) -> None:
        self.client = hvac.Client(url=get_config("vault_url"))
        self.client.token = get_config("vault_token")

    def get_credential(self, identifier: str) -> typing.Dict[str, typing.Any]:
        return self.client.read(identifier)["data"]
"""
